#################################################################################
#!/bin/bash
# Name: worker_security-lockdown_el7.sh
# Description: Security Lockdown script based off of the
#              Center for Internet Security (CIS) Benchmarks Guide
#    Based off of: "CIS Red Hat Enterprise Linux 7 Benchmark v1.1.0"
#    Published on: 04/02/2015
#    Source: http://benchmarks.cisecurity.org
#################################################################################

####========================================
#### OS Version Check - CentOS/Oracle 7 only
####========================================

# Gather distro and major version
if [ -f /etc/system-release-cpe ]; then
  distro=$(awk -F: '{printf "%s", $3}' /etc/system-release-cpe)
  major_version=$(awk -F: '{printf "%d", $5}' /etc/system-release-cpe)
elif [ -f /etc/redhat-release ]; then
  distro=$(awk '{printf "%s", $1}' /etc/redhat-release)
  major_version=$(awk -F. '{print $1}' /etc/redhat-release | awk '{printf "%d", $3}')
fi

# ${distro,,} converts to lower case for comparison
if [[ ${distro,,} == "centos" || ${distro,,} == "oracle" ]]; then
  if [[ ${major_version} != "7" ]]; then
    echo ">>Error: This Security Lockdown script is for CentOS/Oracle Linux 7 only. Exiting..."
    exit 1
  fi
else
  echo ">>Error: This Security Lockdown script is for CentOS/Oracle Linux only. Exiting..."
  exit 1
fi

####=============================================
#### Confirm running the Security Lockdown script
####=============================================
echo -e "======================================================"
echo -e "####========== Security Lockdown Script ==========####"
echo -e "####-----    Enterprise Linux 7.x Version    -----####"
echo -e "======================================================"
echo
echo -e "Warning: This script will remove packages and lock down settings."
echo -e "Detected OS: ${distro} ${major_version}"
echo -e "=>Continue?[y/n]:\c"
read run_script

if [[ ${run_script} != "y" ]]; then
  echo -e "\n>>Will not run the post install script. Exiting..."
  exit 1
fi

#################################################################################
##### Section 1 - Install Updates, Patches and Additional Security Software #####
#################################################################################

echo -e "\n\n###############################################################################"
echo -e "#### Section 1 - Install Updates, Patches and Additional Security Software ####"
echo -e "###############################################################################"
 
####========================================
#### 1.1 - Filesystem Configuration
####========================================
echo -e "\n---- 1.1: File System Configuration ----"

##-- 1.1.1 - Create Separate Partition for /tmp
echo -e "\n>>Checking to see if /tmp has its own partition..."
grep " /tmp " /etc/fstab > /dev/null
if [ $? -eq 1 ]; then
  echo -e "-->Warning: It is recommended that /tmp have its own partition."
else
  echo -e "->Ok: /tmp has its own partition."
fi

##++ Group - nodev/nosuid mount options ++##
##-- 1.1.2 - Set nodev option for /tmp
##-- 1.1.3 - Set nosuid option for /tmp
##-- 1.1.10 - Add nodev option to /home
##-- 1.1.11 - Add nodev option to removable media partitions
##-- 1.1.13 - Add nosuid option to removable media partitions
##-- 1.1.14 - Add nodev option to /dev/shm partition
##-- 1.1.15 - Add nosuid option to /dev/shm partition

# Backup fstab
echo -e "\n>>Backing up /etc/fstab..."
cp -v /etc/fstab /etc/fstab.pre-lockdown

##++ Group - nodev/nosuid mount options ++##
echo -e "\n>>Adding 'nodev' and 'nosuid' to all mounts except / and /usr..."
awk '($3 ~ /^ext[234]$/ && $2 != "/usr" && $2 != "/") { $4 = $4 ",nodev,nosuid" }; { print }' /etc/fstab.pre-lockdown > /etc/fstab

##-- 1.1.4 - Set noexec option for /tmp
echo -e "\n>>Setting 'noexec' to /tmp mount..."
sed -i".tmp" -r -e "s~(^[A-Za-z0-9/=-]+\s+/tmp\s+[a-z0-9]+\s+)[a-z,]+(\s+1 2)~\1defaults,noexec,nosuid,nodev\2~" /etc/fstab

##-- 1.1.5 - Create Separate Partition for /var
echo -e "\n>>Checking to see if /var has its own partition..."
grep " /var " /etc/fstab > /dev/null
if [ $? -eq 1 ]; then
  echo -e "-->Warning: It is recommended that /var have its own partition."
else
  echo -e "->Ok: /var has its own partition."
fi

##-- 1.1.6 - Bind mount the /var/tmp directory to /tmp
echo -e "\n>>Bind mounting the /var/tmp dir to /tmp..."

echo -e "->Removing /var/tmp and remaking it..."
rmdir -v /var/tmp && mkdir -v /var/tmp

echo -e "->Adding bind mount to /etc/fstab..."
echo "/tmp /var/tmp none bind 0 0" >> /etc/fstab

##-- 1.1.7 - Create separate partition for /var/log
echo -e "\n>>Checking to see if /var/log has its own partition..."
grep " /var/log " /etc/fstab > /dev/null
if [ $? -eq 1 ]; then
  echo -e "-->Warning: It is recommended that /var/log have its own partition."
else
  echo -e "->Ok: /var/log has its own partition."
fi

##-- 1.1.8 - Create separate partition for /var/log/audit
echo -e "\n>>Checking to see if /var/log/audit has its own partition..."
grep " /var/log/audit " /etc/fstab > /dev/null
if [ $? -eq 1 ]; then
  echo -e "-->Warning: It is recommended that /var/log/audit have its own partition."
else
  echo -e "->Ok: /var/log/audit has its own partition."
fi

##-- 1.1.9 - Create separate partition for /home
echo -e "\n>>Checking to see if /home has its own partition..."
grep " /home " /etc/fstab > /dev/null
if [ $? -eq 1 ]; then
  echo -e "-->Warning: It is recommended that /home have its own partition."
else
  echo -e "->Ok: /home has its own partition."
fi

##-- 1.1.12 - Add noexec option to removable media partitions
# TODO - Not implemented

##-- 1.1.16 - Add noexec option to /dev/shm partition
echo -e "\n>>Adding 'noexec' option to /dev/shm mount..."
sed -i".tmp" -r -e "s~^(tmpfs\s+/dev/shm\s+tmpfs\s+)[a-z,]+(\s+0 0)~\1defaults,noexec,nosuid,nodev\2~" /etc/fstab

##-- 1.1.17 - Set sticky bit on all world writable directories
# TODO - Not implemented

##++ Group - Disable Filesystems in modprobe ++##
##-- 1.1.18 - Disable mounting of cramfs filesystems
##-- 1.1.19 - Disable mounting of freevxfs filesystems
##-- 1.1.20 - Disable mounting of jffs2 filesystems
##-- 1.1.21 - Disable mounting of hfs filesystems
##-- 1.1.22 - Disable mounting of hfsplus filesystems
##-- 1.1.23 - Disable mounting of squashfs filesystems
##-- 1.1.24 - Disable mounting of udf filesystems
# TODO - Not implemented

####==========================================
#### 1.2 - Configure Software Updates
####==========================================
echo -e "\n---- 1.2: Configure Software Updates ----"

##-- 1.2.1 - Configure connection to the RHN RPM repos
# Skipping - Not applicable for CentOS

##-- 1.2.2 - Verify Red Hat GPG key is installed
# Skipping - Not applicable for CentOS

##-- 1.2.3 - Verify that gpgcheck is globally activated
echo -e "\n>>Verifying that gpgcheck is globally activated..."

# Backup yum.conf
echo -e "->Backing up /etc/yum.conf..."
cp -v /etc/yum.conf /etc/yum.conf.pre-lockdown

echo -e "->Looking for the gpgcheck option..."
grep "^gpgcheck=" /etc/yum.conf > /dev/null

if [ $? -ne 0 ]; then
   # Add 'gpgcheck' to yum.conf
  echo -e "->Adding gpgcheck=1 to yum.conf..."
  sed -i '/^\[main\]/agpgcheck=1' /etc/yum.conf
else
  # Setting exists, just set it to on (1)
  echo -e "->Setting gpgcheck=1 in yum.conf..."
  sed -i -r -e "s/^(gpgcheck=)[01]/\11/" /etc/yum.conf
fi

##-- 1.2.4 - Disable the rhnsd daemon
echo -e "\n>>Will NOT disable rhnsd daemon; this is used for Spacewalk scheduled actions => skipping..."
#daemon="rhnsd"
#echo -e "\n>>Looking for the ${daemon} daemon..."
#systemctl is-enabled ${daemon} > /dev/null 2>&1
#if [ $? -eq 0 ]; then
#  echo -e "->Disabling ${daemon}..."
#  systemctl disable ${daemon}
#else
#  echo -e "->Nothing to do => ${daemon} daemon not found."
#fi

##-- 1.2.5 - Obtain software package updates with yum
# Skipping - yum -y update already run later in this script(1.7).

##-- 1.2.6 - Verify package integrity using rpm
# TODO - Not implemented
# If any output shows, there might be an integrity issue with that package
# rpm --query --verify --all | awk '$2 != "c" { print $0}'

####===================================
#### 1.3 - Instrusion Detection
####===================================
echo -e "\n---- 1.3: Intrusion Detection ----"

echo -e "\n>> Install and configure AIDE => skipping for now..."

##-- 1.3.1 - Install AIDE (Advanced Intrusion Detection)
# TODO - Not implemented
# yum install aide -y

# Initialize AIDE
# TODO - Not implemented
# sed -i s/^PRELINKING=yes/PRELINKING=no/ /etc/sysconfig/prelink
# /usr/sbin/prelink -ua
# /usr/sbin/aide --init -B 'database_out=file:/var/lib/aide/aide.db.gz'

##-- 1.3.2 - Implement periodic execution of file integrity
# TODO - Not implemented
# add a cron job that runs: /usr/sbin/aide --check

####================================
#### 1.4 - Configure SELinux
####================================
echo -e "\n---- 1.4: Configure SELinux ----"

##-- 1.4.1 - Enable SELinux in /etc/grub.conf
# TODO - Not implemented - remove all instances of the following from grub.conf
# selinux=0
# enforcing=0

##-- 1.4.2 - Set the SELinux State
# Disables SELinux (/etc/selinux/config)
echo -e "\n>>Disabling SELinux..."
sed -i "s/^SELINUX=.*/SELINUX=disabled/" /etc/selinux/config
sed -i "s/^SELINUX=.*/SELINUX=disabled/" /etc/sysconfig/selinux

##-- 1.4.3 - Set the SELinux Policy
# Skipping - Not applicable, SELinux disabled
# Edit /etc/selinux/config
# Set 'SELINUXTYPE=' to desired policy

##-- 1.4.4 - Remove SETroubleshoot
echo -e "\n>>Checking for setroubleshoot..."
rpm -q setroubleshoot
if [ $? -eq 0 ]; then
  echo -e "\n->Removing setroubleshoot..."
  yum -y remove setroubleshoot
fi

##-- 1.4.5 - Remove MCS translation service
echo -e "\n>>Checking for mcstrans..."
rpm -q mcstrans
if [ $? -eq 0 ]; then
  echo -e "\n->Removing mcstrans..."
  yum -y remove mcstrans
fi

##-- 1.4.6 - Check for unconfirmed daemons
# Skipping - Not applicable, SELinux enforcing dependent

####===================================
#### 1.5 - Secure Boot Settings
####===================================
echo -e "\n---- 1.5: Secure Boot Settings ----"

##-- 1.5.1 - Set user/group owner on grub.cfg
echo -e "\n>>Setting user/group owner on /boot/grub2/grub.cfg..."
chown -v root:root /boot/grub2/grub.cfg

##-- 1.5.2 - Set permissions on grub.cfg
echo -e "\n>>Setting permissions on /boot/grub2/grub.cfg..."
chmod -v 600 /boot/grub2/grub.cfg

##-- 1.5.3 Set boot loader password
# TODO - Not Implemented
#echo -e "\n>>Checking for grub boot loader password..."
#grep "^set superusers" /boot/grub2/grub.cfg
#grep "^password" /boot/grub2/grub.cfg
#if [ $? -eq 1 ]; then
#  echo "->Setting grub boot loader password..."
#  grub2-mkpasswd-pbkdf2
#  create custom /etc/grub.d config file:
#  cat <<EOF
#  set superusers="<user-list>"
#  password_pbkdf2 <user> <encrypted-password>
#  EOF
#  grub2-mkconfig -o /boot/grub2/grub.cfg
#else
#  echo "->Grub password already exists in /boot/grub2/grub.cfg."
#fi

####===========================================
#### 1.6 - Additional Process Hardening
####===========================================
echo -e "\n---- 1.6: Additional Process Hardening ----"

##-- 1.6.1 - Restrict core dumps 
echo -e "\n>>Restrict core dumps => Handled in sysctl.conf and limits.conf section..."

##-- 1.6.2 - Enable randomized virtual memory region placement
echo -e "\n>>Enabled randomized virtual memory region => Handled in sysctl.conf section..."

####===========================================
#### 1.7 - Use the Latest OS Release
####===========================================
echo -e "\n---- 1.7: Use the Latest OS Release ----"
echo -e "\n>>Checking for system updates..."
yum -y update

#################################################################################
#####                    Section 2 - OS Services                            #####
#################################################################################
echo -e "\n\n#################################"
echo -e "#### Section 2 - OS Services ####"
echo -e "#################################"

####=======================================
#### 2.1 - Remove Legacy Services
####=======================================
echo -e "\n---- 2.1: Remove Legacy Services ----"

echo -e "\n>>Checking for legacy services..."

##-- 2.1.1 - Remove telnet server
rpm -q telnet-server
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing telnet-server..."
 yum -y remove telnet-server
fi

##-- 2.1.2 - Remove telnet client
rpm -q telnet
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing telnet client..."
 yum -y remove telnet
fi

##-- 2.1.3 - Remove rsh-server
rpm -q rsh-server
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing rsh-server..."
 yum -y remove rsh-server
fi

##-- 2.1.4 - Remove rsh
rpm -q rsh
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing rsh..."
 yum -y remove rsh
fi

##-- 2.1.5 - Remove NIS client
rpm -q ypbind
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing ypbind..."
 yum -y remove ypbind
fi

##-- 2.1.6 - Remove NIS server
rpm -q ypserv
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing ypserv..."
 yum -y remove ypserv
fi

##-- 2.1.7 - Remove tftp
rpm -q tftp
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing tftp client..."
 yum -y remove tftp
fi

##-- 2.1.8 - Remove tftp-server
rpm -q tftp-server
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing tftp-server..."
 yum -y remove tftp-server
fi

##-- 2.1.9 - Remove talk
rpm -q talk
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing talk..."
 yum -y remove talk
fi

##-- 2.1.10 - Remove talk-server
rpm -q talk-server
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing talk-server..."
 yum -y remove talk-server
fi

##-- 2.1.11 - Remove xinetd
rpm -q xinetd
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing xinetd..."
 yum -y remove xinetd
fi

##-- 2.1.12 - Disable chargen-dgram
daemon="chargen-dgram"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.13 - Disable chargen-stream
daemon="chargen-stream"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.14 - Disable daytime-dgram
daemon="daytime-dgram"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.15 - Disable daytime-stream
daemon="daytime-stream"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.16 - Disable echo-dgram
daemon="echo-dgram"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.17 - Disable echo-stream
daemon="echo-stream"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

##-- 2.1.18 - Disable tcpmux-server
daemon="tcpmux-server"
echo -e "\n>>Looking for the ${daemon} daemon..."
chkconfig --list ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  chkconfig ${daemon} off
else
  echo -e "->Nothing to do => ${daemon} daemon not found."
fi

#################################################################################
#####                Section 3 - Special Purpose Services                   #####
#################################################################################
echo -e "\n\n##############################################"
echo -e "#### Section 3 - Special Purpose Services ####"
echo -e "##############################################"

####=======================================
#### 3.1 - Set Daemon umask
####=======================================
echo -e "\n---- 3.1: Set Daemon umask ----"

egrep "^umask [0-9]{3}" /etc/sysconfig/init > /dev/null

if [ $? -ne 0 ]; then
  echo -e "\n>>Setting daemon mask to 027..."
  echo -e "# Default daemon umask\numask 027" >> /etc/sysconfig/init
else
  echo -e "\n>>Daemon umask already set in /etc/sysconfig/init to: $(grep "^umask" /etc/sysconfig/init)."
fi

####=======================================
#### 3.2 - Remove X-Windows
####=======================================
echo -e "\n---- 3.2: Remove X-Windows ----"

echo -e "\n>> Setting default boot target to multi-user without GUI..."
systemctl set-default multi-user.target

echo -e "\n>>Uninstall X Window Server..."
rpm -q xorg-x11-server-common
if [ $? -eq 0 ]; then
  echo -e "\n>>Removing X Window Server..."
  yum -y remove xorg-x11-server-common
fi

####=======================================
#### 3.3 - Disable Avahi Server
####=======================================
echo -e "\n---- 3.3: Disable Avahi Server ----"

daemon="avahi-daemon"
echo -e "\n>>Looking for the ${daemon} daemon..."
systemctl is-enabled ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  systemctl disable ${daemon}
else
  echo -e "->Nothing to do => ${daemon} daemon not found or already disabled."
fi

####=======================================
#### 3.4 - Disable Print Server - CUPS
####=======================================
echo -e "\n---- 3.4: Disable Print Server - CUPS ----"

daemon="cups"
echo -e "\n>>Looking for the ${daemon} daemon..."
systemctl is-enabled ${daemon} > /dev/null 2>&1
if [ $? -eq 0 ]; then
  echo -e "->Disabling ${daemon}..."
  systemctl disable ${daemon}
else
  echo -e "->Nothing to do => ${daemon} daemon not found or already disabled."
fi

####=======================================
#### 3.5 - Remove DHCP Server
####=======================================
echo -e "\n---- 3.5: Remove DHCP Server ----"

rpm -q dhcp
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing dhcp server..."
 yum -y remove dhcp
fi

####=======================================
#### 3.6 - Configure NTP services
####=======================================
echo -e "\n---- 3.6: Configure NTP Services ----"
echo -e "\n>>NTP already configured in postinstall.sh => skipping..."

####=======================================
#### 3.7 - Remove LDAP
####=======================================
echo -e "\n---- 3.7: Remove LDAP ----"
echo -e "\n>>LDAP in use => skipping..."

####=======================================
#### 3.8 - Disable NFS and RPC
####=======================================
echo -e "\n---- 3.8: Disable NFS and RPC ----"
echo -e "\n>>NFS in use => skipping..."

####=======================================
#### 3.9 - Remove DNS server
####=======================================
echo -e "\n---- 3.9: Remove DNS Server ----"
rpm -q bind
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing bind..."
 yum -y remove bind
fi

####=======================================
#### 3.10 - Remove FTP server
####=======================================
echo -e "\n---- 3.10: Remove FTP Server ----"
rpm -q vsftpd
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing vsftpd..."
 yum -y remove vsftpd
fi

####=======================================
#### 3.11 - Remove HTTP server
####=======================================
echo -e "\n---- 3.11: Remove HTTP Server ----"
rpm -q httpd
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing httpd..."
 yum -y remove httpd
fi

####=======================================
#### 3.12 - Remove Dovecot(IMAP/POP3)
####=======================================
echo -e "\n---- 3.12: Remove Dovecot(IMAP/POP3) ----"
rpm -q dovecot
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing dovecot..."
 yum -y remove dovecot
fi

####=======================================
#### 3.13 - Remove Samba
####=======================================
echo -e "\n---- 3.13: Remove Samba ----"
rpm -q samba
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing samba..."
 yum -y remove samba
fi

####=======================================
#### 3.14 - Remove HTTP Proxy Server
####=======================================
echo -e "\n---- 3.14: Remove HTTP Proxy Server ----"
rpm -q squid
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing squid..."
 yum -y remove squid
fi

####=======================================
#### 3.15 - Remove SNMP Server
####=======================================
echo -e "\n---- 3.15: Remove SNMP Server ----"
rpm -q net-snmp
if [ $? -eq 0 ]; then
 echo -e "\n>>Removing net-snmp..."
 yum -y remove net-snmp
fi

####==========================================================
#### 3.16 - Configure Mail Transfer Agent for Local-Only Mode
####==========================================================
echo -e "\n---- 3.16: Configure MTA for Local-Only Mode ----"

echo -e "\n>>Looking for postfix..."
rpm -q postfix
if [ $? -eq 0 ]; then
 echo -e "->Configuring postfix to listen on localhost only..."
 sed -i -r -e "s/(^inet_interfaces =).+/\1 localhost/" /etc/postfix/main.cf
else
  echo -e "-->Warning: postfix not found."
fi

#################################################################################
#####            Section 4 - Network Configuration and Firewalls            #####
#################################################################################
echo -e "\n\n#########################################################"
echo -e "#### Section 4 - Network Configuration and Firewalls ####"
echo -e "#########################################################"

####==========================================================
#### 4.1 - Modify Network Parameters(Host Only)
####==========================================================
echo -e "\n---- 4.1: Modify Network Parameters(Host Only) ----"

##-- 4.1.1 - Disable IP fowarding
# net.ipv4.ip_forward=0
echo -e "\n>>Disable IP forwarding => Handled in sysctl.conf section..."

##-- 4.1.2 - Disable send packet redirects
# net.ipv4.conf.all.send_redirects=0 
# net.ipv4.conf.default.send_redirects=0
echo -e "\n>>Disable send packet redirects => Handled in sysctl.conf section..."

####======================================================
#### 4.2 - Modify Network Parameters(Host and Router Only)
####======================================================
echo -e "\n---- 4.2: Modify Network Parameters(Host and Router Only) ----"

##-- 4.2.1 - Disable source routed packet acceptance
# net.ipv4.conf.all.accept_source_route=0
# net.ipv4.conf.default.accept_source_route=0
echo -e "\n>>Disable source routed packet acceptance => Handled in sysctl.conf section..."

##-- 4.2.2 - Disable ICMP redirect acceptance
# net.ipv4.conf.all.accept_redirects=0 
# net.ipv4.conf.default.accept_redirects=0
echo -e "\n>>Disable ICMP redirect acceptance => Handled in sysctl.conf section..."

##-- 4.2.3 - Disable secure ICMP redirect acceptance
# net.ipv4.conf.all.secure_redirects=0
# net.ipv4.conf.default.secure_redirects=0
echo -e "\n>>Disable secure ICMP redirect acceptance => Handled in sysctl.conf section..."

##-- 4.2.4 - Log suspicious packets
# net.ipv4.conf.all.log_martians=1
# net.ipv4.conf.default.log_martians=1
echo -e "\n>>Log suspicious packets => Handled in sysctl.conf section..."

##-- 4.2.5 - Enable ignore broadcast requests
# net.ipv4.icmp_echo_ignore_broadcasts=1
echo -e "\n>>Enable ignore broadcast requests => Handled in sysctl.conf section..."

##-- 4.2.6 - Enable bad error message protection
# net.ipv4.icmp_ignore_bogus_error_responses=1
echo -e "\n>>Enable bad error message protection => Handled in sysctl.conf section..."

##-- 4.2.7 - Enable RFC recommended source route validation
# net.ipv4.conf.all.rp_filter=1
# net.ipv4.conf.default.rp_filter=1
echo -e "\n>>Enable RFC recommended source route validation => Handled in sysctl.conf section..."

##-- 4.2.8 - Enable TCP SYN cookies
# net.ipv4.tcp_syncookies=1
echo -e "\n>>Enable TCP SYN cookies => Handled in sysctl.conf section..."

####==================================
#### 4.3 - Wireless Networking
####==================================
echo -e "\n---- 4.3: Wireless Networking ----"

##-- 4.3.1 - Deactive wireless interfaces
echo -e "\n>>Deactivate wireless interfaces not applicable => skipping..."

####=============================
#### 4.4 - IPv6
####=============================
echo -e "\n---- 4.4: IPv6 ----"

# TODO - Not implemented...need to either configure IPv6 or Disable it.

echo -e "\n>>Configure IPv6 => skipping for now..."
##-- 4.4.1 - Configure IPv6
# If in use, configure using these benchmarks

#- 4.4.1.1 - Disable IPv6 Router Advertisements
# /etc/sysctl.conf
# net.ipv6.conf.all.accept_ra=0
# net.ipv6.conf.default.accept_ra=0

#- 4.4.1.2 - Disable IPv6 redirect acceptance
# /etc/sysctl.conf
# net.ipv6.conf.all.accept_redirects=0
# net.ipv6.conf.default.accept_redirects=0

##-- 4.4.2 - Disable IPv6
# If no in use, disable to reduce attack surface
# /etc/sysctl.conf
# net.ipv6.conf.all.disable_ipv6=1

####====================================
#### 4.5 - Install TCP Wrappers
####====================================
echo -e "\n---- 4.5: Install TCP Wrappers ----"

##-- 4.5.1 - Install TCP wrappers
echo -e "\n>>Install TCP wrappers => skipping for now..."
# rpm -q tcp_wrappers
# if [ $? -eq 0 ]; then
#   yum -y install tcp_wrappers
# fi

##-- 4.5.2 - Create /etc/hosts.allow
# TODO - Not implemented
# echo "ALL: <net>/<mask>, <net>/<mask>, …" >>/etc/hosts.allow

##-- 4.5.3 - Verify permissions on /etc/hosts.allow
echo -e "\n>>Verifying permissions on /etc/hosts.allow..."
chmod -v 644 /etc/hosts.allow

##-- 4.5.4 - Create /etc/hosts.deny
# TODO - Not implemented
# grep "ALL: ALL" /etc/hosts.deny
# if [ $? -eq 1 ]; then
#   echo "ALL: ALL" >> /etc/hosts.deny
# fi

##-- 4.5.5 - Verify permissions on /etc/hosts.deny
echo -e "\n>>Verifying permissions on /etc/hosts.deny..."
chmod -v 644 /etc/hosts.deny

####==========================================
#### 4.6 - Uncommon Network Protocols
####==========================================
echo -e "\n---- 4.6: Uncommon Network Protocols ----"

echo -e "\n>>Disable uncommon network protocols => skipping for now..."
##-- 4.6.1 - Disable DCCP
# TODO - Not implemented
# grep "install dccp /bin/true" /etc/modprobe.d/lockdown.conf
# echo "install dccp /bin/true" >> /etc/modprobe.d/lockdown.conf

##-- 4.6.2 - Disable SCTP
# TODO - Not implemented
# grep "install sctp /bin/true" /etc/modprobe.d/lockdown.conf
# echo "install sctp /bin/true" >> /etc/modprobe.d/lockdown.conf

##-- 4.6.3 - Disable RDS
# TODO - Not implemented
# grep "install rds /bin/true" /etc/modprobe.d/lockdown.conf
# echo "install rds /bin/true" >> /etc/modprobe.d/lockdown.conf

##-- 4.6.4 - Disable TIPC
# TODO - Not implemented
# grep "install tipc /bin/true" /etc/modprobe.d/lockdown.conf
# echo "install tipc /bin/true" >> /etc/modprobe.d/lockdown.conf

####==========================================
#### 4.7 - Enable Firewalld
####==========================================
echo -e "\n---- 4.7: Enable Firewalld ----"

echo -e "\n>>Disables firewalld => skipping..."
#systemctl is-enabled firewalld
#if [ $? -ne 0 ]; then
# systemctl enable firewalld
#fi

#################################################################################
#####                   Section 5 - Logging and Auditing                    #####
#################################################################################
echo -e "\n\n##########################################"
echo -e "#### Section 5 - Logging and Auditing ####"
echo -e "##########################################"

####=================================
#### 5.1 - Configure rsyslog
####=================================
echo -e "\n---- 5.1: Configure Rsyslog ----"

##-- 5.1.1 - Install the rsyslog package
echo -e "\n>>Install rsyslog completed in postinstall.sh => skipping..."

##-- 5.1.2 - Activate the rsyslog service
echo -e "\n>>Activate rsyslog completed in postinstall.sh => skipping..."

##-- 5.1.3 - Configure /etc/rsyslog.conf
echo -e "\n>>Configure rsyslog completed in postinstall.sh => skipping..."

##-- 5.1.4 - Create and set permissions on rsyslog log files
# TODO - Not implemented
# set all rsyslog log files to
# ownership => root:root
# permissions => 600

##-- 5.1.5 - Configure rsyslog to send logs to a remote log host
echo -e "\n>>Configure rsyslog to send logs to a remote log host completed in postinstall.sh => skipping..."

##-- 5.1.6 - Accept remote rsyslog messages only on designated log hosts
echo -e "\n>>Accept remote rsyslog messages only on log hosts completed in postinstall.sh => skipping..."

####===================================================
#### 5.2 - Configure System Accounting(auditd)
####===================================================
echo -e "\n---- 5.2: Configure System Accounting(auditd) ----"

##-- 5.2.1 - Configure data retention

#- 5.2.1.1 - Configure audit log storage size
# TODO - Not implemented
# /etc/audit/auditd.conf
# max_log_file = 5

#- 5.2.1.2 - Disable system on audit log full
# TODO - Not implemented
# /etc/audit/auditd.conf
# space_left_action = email
# action_mail_acct = root
# admin_space_left_action = halt

#- 5.2.1.3 - Keep all auditing information
# TODO - Not implemented
# /etc/audit/auditd.conf
# max_log_file_action = keep_logs

##-- 5.2.2 - Enable auditd service
echo -e "\n>>Enabling auditd..."
systemctl enable auditd

##-- 5.2.3 - Enable auditing for processes that start prior to auditd
echo -e "\n>>Enabling auditing for processes that start prior to auditd..."

# Remove trailing quote (") of GRUB_CMDLINE_LINUX line
sed -i -r -e "/^GRUB_CMDLINE_LINUX/s/\"$//" /etc/default/grub

# Add audit=1" to the end of line
sed -i -r -e "/^GRUB_CMDLINE_LINUX/s/^(GRUB_CMDLINE_LINUX=\".*)/\1 audit=1\"/" /etc/default/grub

# Update grub2 config
grub2-mkconfig -o /boot/grub2/grub.cfg

# TODO - Not implemented

##-- 5.2.4 - Record events that modify date and time information
#echo -e "\n>>Auditd => record events that modify date and time information..."
#grep time-change /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Record events that modify date and time information" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S adjtimex -S settimeofday -k time-change" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S adjtimex -S settimeofday -S stime -k time-change" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S clock_settime -k time-change" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S clock_settime -k time-change" >> /etc/audit/audit.rules
#  echo "-w /etc/localtime -p wa -k time-change" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.5 - Record events that modify user/group information
# echo -e "\n>>Auditd => record events that modify user/group info..."
#grep identity /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Record events that modify user/group information" >> /etc/audit/audit.rules
#  echo "-w /etc/group -p wa -k identity" >> /etc/audit/audit.rules
#  echo "-w /etc/passwd -p wa -k identity" >> /etc/audit/audit.rules
#  echo "-w /etc/gshadow -p wa -k identity" >> /etc/audit/audit.rules
#  echo "-w /etc/shadow -p wa -k identity" >> /etc/audit/audit.rules
#  echo "-w /etc/security/opasswd -p wa -k identity" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.6 - Record events that modify the system's network environment
#echo -e "\n>>Auditd => record events that modify system network env..."
#grep system-locale /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Record events that modify the system's network environment" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S sethostname -S setdomainname -k system-locale" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S sethostname -S setdomainname -k system-locale" >> /etc/audit/audit.rules
#  echo "-w /etc/issue -p wa -k system-locale" >> /etc/audit/audit.rules
#  echo "-w /etc/issue.net -p wa -k system-locale" >> /etc/audit/audit.rules
#  echo "-w /etc/hosts -p wa -k system-locale" >> /etc/audit/audit.rules
#  echo "-w /etc/sysconfig/network -p wa -k system-locale" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.7 - Record events that modify the system's mandatory access controls
#echo -e "\n>>Auditd => record events that modify system mandatory access controls..."
#grep MAC-policy /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Record events that modify the system's mandatory access controls" >> /etc/audit/audit.rules
#  echo "-w /etc/selinux/ -p wa -k MAC-policy" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.8 - Collect login and logout events
#echo -e "\n>>Auditd => collect login and logout events..."
#grep logins /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect login and logout events" >> /etc/audit/audit.rules
#  echo "-w /var/log/faillog -p wa -k logins" >> /etc/audit/audit.rules
#  echo "-w /var/log/lastlog -p wa -k logins" >> /etc/audit/audit.rules
#  echo "-w /var/log/tallylog -p wa -k logins" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.9 - Collect session initiation information
#echo -e "\n>>Auditd => collect session initiation info..."
#grep session /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect session initiation information" >> /etc/audit/audit.rules
#  echo "-w /var/run/utmp -p wa -k session" >> /etc/audit/audit.rules
#  echo "-w /var/log/wtmp -p wa -k session" >> /etc/audit/audit.rules
#  echo "-w /var/log/btmp -p wa -k session" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.10 - Collect discretionary access control permission modification events
#echo -e "\n>>Auditd => collect discretionary access control events..."
#grep perm_mod /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect discretionary access control permission modification events" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S chmod -S fchmod -S fchmodat -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S chmod -S fchmod -S fchmodat -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S chown -S fchown -S fchownat -S lchown -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S chown -S fchown -S fchownat -S lchown -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S setxattr -S lsetxattr -S fsetxattr -S removexattr -S lremovexattr -S fremovexattr -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S setxattr -S lsetxattr -S fsetxattr -S removexattr -S \ lremovexattr -S fremovexattr -F auid>=500 -F auid!=4294967295 -k perm_mod" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.11 - Collect unsuccessful unauthorized access attempts to files
#echo -e "\n>>Auditd => collect unsuccessful unauthorized access attempts to files..."
#grep access /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect unsuccessful unauthorized access attempts to files" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S creat -S open -S openat -S truncate -S ftruncate -F exit=-EACCES -F auid>=500 -F auid!=4294967295 -k access" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S creat -S open -S openat -S truncate -S ftruncate -F exit=-EACCES -F auid>=500 -F auid!=4294967295 -k access" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S creat -S open -S openat -S truncate -S ftruncate -F exit=-EPERM -F auid>=500 -F auid!=4294967295 -k access" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S creat -S open -S openat -S truncate -S ftruncate -F exit=-EPERM -F auid>=500 -F auid!=4294967295 -k access" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.12 - Collect use of privileged commands
# verify that an audit line for each setuid/setgid program appears in the audit file
# find PART -xdev \( -perm -4000 -o -perm -2000 \) -type f | awk '{print \ "-a always,exit -F path=" $1 " -F perm=x -F auid>=500 -F auid!=4294967295 \ -k privileged" }'
# add the above lines to /etc/audit/audit.rules

##-- 5.2.13 - Collect successful file system mounts
#echo -e "\n>>Auditd => collect succcessful file system mounts..."
#grep mounts /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect successful file system mounts" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S mount -F auid>=500 -F auid!=4294967295 -k mounts" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S mount -F auid>=500 -F auid!=4294967295 -k mounts" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.14 - Collect file deletion events by user
#echo -e "\n>>Auditd => collect file deletion events by user..."
#grep delete /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect file deletion events by user" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S unlink -S unlinkat -S rename -S renameat -F auid>=500 -F auid!=4294967295 -k delete" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b32 -S unlink -S unlinkat -S rename -S renameat -F auid>=500 -F auid!=4294967295 -k delete" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.15 - Collect changes to system administration scope
#echo -e "\n>>Auditd => collect changes to system admin scope..."
#grep scope /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect changes to system administration scope" >> /etc/audit/audit.rules
#  echo "-w /etc/sudoers -p wa -k scope" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.16 - Collect system administrator actions (sudolog)
#echo -e "\n>>Auditd => collect system admin actions..."
#grep actions /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect system administrator actions (sudolog)" >> /etc/audit/audit.rules
#  echo "-w /var/log/sudo.log -p wa -k actions" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.17 - Collect kernel module loading and unloading
#echo -e "\n>>Auditd => collect kernel module loading and unloading..."
#grep modules /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Collect kernel module loading and unloading" >> /etc/audit/audit.rules
#  echo "-w /sbin/insmod -p x -k modules" >> /etc/audit/audit.rules
#  echo "-w /sbin/rmmod -p x -k modules" >> /etc/audit/audit.rules
#  echo "-w /sbin/modprobe -p x -k modules" >> /etc/audit/audit.rules
#  echo "-a always,exit -F arch=b64 -S init_module -S delete_module -k modules" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

##-- 5.2.18 - Make the audit configuration immutable
#echo -e "\n>>Auditd => make config immutable..."
#grep "^-e 2" /etc/audit/audit.rules
#if [ $? -eq 1 ]; then
#  echo -e "\n# Make the audit configuration immutable" >> /etc/audit/audit.rules
#  echo "-e 2" >> /etc/audit/audit.rules
#  pkill -HUP -P 1 auditd
#fi

####===================================================
#### 5.3 - Configure logrotate
####===================================================
echo -e "\n---- 5.3: Configure logrotate ----"

echo -e "\n>>Ensuring system logs are rotated..."

for FILE in /var/log/messages /var/log/secure /var/log/maillog /var/log/spooler /var/log/boot.log /var/log/cron; do
  grep "${FILE}" /etc/logrotate.d/syslog > /dev/null
  if [ $? -eq 1 ]; then
    echo "->Adding ${FILE} to /etc/logrotate.d/syslog..."
    sed -i "1i ${FILE}" /etc/logrotate.d/syslog
  fi
done

#################################################################################
#####      Section 6 - System Access, Authentication, and Authorization     #####
#################################################################################
echo -e "\n\n######################################################################"
echo -e "#### Section 6 - System Access, Authentication, and Authorization ####"
echo -e "######################################################################"

####==========================================
#### 6.1 - Configure cron and anacron
####==========================================
echo -e "\n---- 6.1: Configure Cron and Anacron ----"

##-- 6.1.1 - Enable anacron daemon
echo -e "\n>>Enable anacron on systems that are not up 24x7 => skipping..."
# rpm -q cronie-anacron
#if [ $? -eq 1 ]; then
#  yum -y install cronie-anacron
#fi

##-- 6.1.2 - Enable crond deamon
echo -e "\n>>Enabling crond..."
systemctl enable crond

##-- 6.1.3 - Set user/group owner and permission on /etc/anacrontab
echo -e "\n>>Setting user/group owner and permissions on anacrontab..."
chown -v root:root /etc/anacrontab
chmod -v 600 /etc/anacrontab

##-- 6.1.4 - Set user/group owner and permission on /etc/crontab
echo -e "\n>>Setting user/group owner and permissions on crontab..."
chown -v root:root /etc/crontab
chmod -v 600 /etc/crontab

##-- 6.1.5 - Set user/group owner and permission on /etc/cron.hourly
echo -e "\n>>Setting user/group owner and permissions on cron.hourly..."
chown -v root:root /etc/cron.hourly
chmod -v go-rwx /etc/cron.hourly

##-- 6.1.6 - Set user/group owner and permission on /etc/cron.daily
echo -e "\n>>Setting user/group owner and permissions on cron.daily..."
chown -v root:root /etc/cron.daily
chmod -v go-rwx /etc/cron.daily

##-- 6.1.7 - Set user/group owner and permission on /etc/cron.weekly
echo -e "\n>>Setting user/group owner and permissions on cron.weekly..."
chown -v root:root /etc/cron.weekly
chmod -v go-rwx /etc/cron.weekly

##-- 6.1.8 - Set user/group owner and permission on /etc/cron.monthly
echo -e "\n>>Setting user/group owner and permissions on cron.monthly..."
chown -v root:root /etc/cron.monthly
chmod -v go-rwx /etc/cron.monthly

##-- 6.1.9 - Set user/group owner and permission on /etc/cron.d
echo -e "\n>>Setting user/group owner and permissions on cron.d..."
chown -v root:root /etc/cron.d
chmod -v go-rwx /etc/cron.d

##-- 6.1.10 - Restrict at daemon
echo -e "\n>>Restricting at daemon..."
rm -v /etc/at.deny
echo "root" > /etc/at.allow
chown -v root:root /etc/at.allow
chmod -v 600 /etc/at.allow

##-- 6.1.11 - Restrict at/cron to authorized users
echo -e "\n>>Restricting cron to authorized users..."
rm -v /etc/cron.deny
echo "root" > /etc/cron.allow
chown -v root:root /etc/cron.allow
chmod -v 600 /etc/cron.allow

####==============================
#### 6.2 - Configure SSH
####==============================
echo -e "\n---- 6.2: Configure SSH Server ----"

# Backup config
echo -e "\n>>Backing up sshd_config..."
cp -v /etc/ssh/sshd_config /etc/ssh/sshd_config.pre-lockdown

##-- 6.2.1 - Set ssh protocol to 2
echo -e "\n>>Setting SSH protocol to 2 only..."
sed -i "/Protocol/s/^#*Protocol 2,1/Protocol 2/" /etc/ssh/sshd_config

##-- 6.2.2 - Set loglevel to info
echo -e "\n>>Setting SSH loglevel to info..."
sed -i "s/^#LogLevel/LogLevel/;s/^LogLevel [A-Za-z]*/LogLevel INFO/" /etc/ssh/sshd_config

##-- 6.2.3 - Set permissions on /etc/ssh/sshd_config
echo -e "\n>>Setting permissions on /etc/ssh/sshd_config..."
chown -v root:root /etc/ssh/sshd_config
chmod -v 600 /etc/ssh/sshd_config

##-- 6.2.4 - Disable SSH x11 forwarding
echo -e "\n>>Disable X11 forwarding..."
sed -i "s/^#X11Forwarding/X11Forwarding/;s/^X11Forwarding [:alpha:]/X11Forwarding no/" /etc/ssh/sshd_config

##-- 6.2.5 - Set ssh MaxAuthTries to 4 or less
echo -e "\n>>Setting ssh MaxAuthTries to 3..."
sed -i -r -e "/MaxAuthTries/s/#*(MaxAuthTries) [0-9]{1}/\1 3/" /etc/ssh/sshd_config

##-- 6.2.6 - Set ssh IgnoreRhosts to Yes
echo -e "\n>>SSH IgnoreRhosts to Yes..."
sed -i -r -e "/IgnoreRhosts/s/#*(IgnoreRhosts yes)/\1/" /etc/ssh/sshd_config

##-- 6.2.7 - Set ssh HostBasedAuthentication to no
echo -e "\n>>SSH HostBasedAuthentication to no..."
sed -i -r -e "/HostbasedAuthentication \w+/s/#*(HostbasedAuthentication no)/\1/" /etc/ssh/sshd_config

##-- 6.2.8 - Disable ssh root login
echo -e "\n>>Disable SSH PermitRootLogin..."
sed -i -r -e "/PermitRootLogin/s/#*(PermitRootLogin )\w+/\1 yes/" /etc/ssh/sshd_config

##-- 6.2.9 - Set ssh PermitEmptyPasswords to no
echo -e "\n>>SSH PermitEmptyPasswords to no..."
sed -i -r -e "/PermitEmptyPasswords/s/#*(PermitEmptyPasswords) \w+/\1 no/" /etc/ssh/sshd_config

##-- 6.2.10 - Do not allow users to set environment options
echo -e "\n>>SSH PermitUserEnvironments to no..."
sed -i -r -e "/PermitUserEnvironment/s/#*(PermitUserEnvironment) \w+/\1 no/" /etc/ssh/sshd_config

##-- 6.2.11 - Use only approved cipher in counter mode
echo -e "\n>>SSH use only approved ciphers..."
grep -q "^Ciphers" /etc/ssh/sshd_config > /dev/null 2>&1
if [ $? -eq 0 ]; then
  sed -i -r -e "/Ciphers/s/#*(Ciphers) .*/\1 aes128-ctr,aes192-ctr,aes256-ctr/" /etc/ssh/sshd_config
else
  echo "Ciphers aes128-ctr,aes192-ctr,aes256-ctr" >> /etc/ssh/sshd_config
fi

##-- 6.2.12 - Set idle timeout interval for user login
echo -e "\n>>Setting SSH idle timeout..."
# ClientAliveInterval => How many seconds before sending a keep alive to an idle SSH session (300 = 5 mins)
# ClientAliveCountMax => How many keep alives to send before disconnecting
sed -i "s/^#ClientAliveInterval/ClientAliveInterval/;s/^ClientAliveInterval [0-9]*/ClientAliveInterval 300/" /etc/ssh/sshd_config
sed -i "s/^#ClientAliveCountMax/ClientAliveCountMax/;s/^ClientAliveCountMax [0-9]*/ClientAliveCountMax 0/" /etc/ssh/sshd_config

##-- 6.2.13 - Limit access via ssh
# TODO - Not implemented
# comma separated list
# /etc/ssh/sshd_config
# AllowGroups <grouplist>

##-- 6.2.14 - Set ssh banner   
echo -e "\n>>Setting banner to use /etc/issue..."
sed -i "/Banner/s/#*Banner\ none/Banner\ \/etc\/issue/" /etc/ssh/sshd_config

####===============================================================
#### 6.3 - Configure PAM (Pluggable Authentication Modules)
####===============================================================
echo -e "\n---- 6.3: Configure PAM ----"

##-- 6.3.1 - Upgrade password hashing algorithm to SHA-512

# Setting password algorithm for local accts to sha512
authconfig --test | grep hashing | grep sha512 > /dev/null 2>&1
if [ $? -ne 0 ]; then
  echo -e "\n>>Setting password algorithm for local accounts to SHA512..."
  authconfig --passalgo=sha512 --update > /dev/null 2>&1
fi

##-- 6.3.2 - Set password creation requirement paramater using pam_cracklib
# TODO - Not implemented
#authconfig --test | grep -qi pam_ldap | grep -q enabled > /dev/null 2>&1
#if [ $? -ne 0 ]; then
#  echo -e "\n>>Setting local password creation requirement..."
#  sed -i -r -e "/pam_cracklib.so/s/^(password\s+\w+\s+pam_cracklib.so).*/\1 try_first_pass retry=3 minlen=12,dcredit=-1,ucredit=-1,ocredit=-1 lcredit=-1/" /etc/pam.d/system-auth-ac
#fi

##-- 6.3.3 - Set lockout for failed password attempts
# TODO - Not implemented

# /etc/pam.d/password-auth
#auth required pam_env.so
#auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900
#auth [success=1 default=bad] pam_unix.so
#auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900
#auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900
#auth required pam_deny.so

# /etc/pam.d/system-auth
#auth required pam_env.so
#auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900
#auth [success=1 default=bad] pam_unix.so
#auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900
#auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900
#auth required pam_deny.so

##-- 6.3.4 - Limit password reuse
# TODO - Not implemented
# Local accounts only
# /etc/pam.d/system-auth
# password sufficient pam_unix.so remember=5

####=====================================================
#### 6.4 - Restrict root login to system console
####=====================================================
echo -e "\n---- 6.4: Restrict Root Login to System Console ----"

echo -e "\n>>Setting consoles in /etc/securetty..."
cat > /etc/securetty <<EOF
console
vc/1
vc/2
vc/3
vc/4
vc/5
vc/6
tty1
tty2
tty3
tty4
tty5
tty6
EOF

####=========================================
#### 6.5 - Restrict Access to the su Command
####=========================================
echo -e "\n---- 6.5: Restrict Access to the su Command ----"

echo -e "\n>>Skipping for now..."
# TODO - Not implemented
# Requires adding individual users to wheel group
# /etc/pam.d/su
# auth required pam_wheel.so use_uid
# Then add users to the wheel group

#################################################################################
#####              Section 7 - User Accounts and Environment                #####
#################################################################################
echo -e "\n\n###################################################"
echo -e "#### Section 7 - User Accounts and Environment ####"
echo -e "###################################################"

####============================================
#### 7.1 - Set Shadow Password Suite Parameters
####============================================
echo -e "\n---- 7.1: Set Shadow Password Suite Paramters ----"

# backup config
echo -e "\n>>Backing up /etc/login.defs..."
cp -v /etc/login.defs /etc/login.defs.pre-lockdown

##-- 7.1.1 - Set password expiration days
echo -e "\n>>Password expiration max days to 90..."
sed -i 's/^PASS_MAX_DAYS.*/PASS_MAX_DAYS  90/' /etc/login.defs

##-- 7.1.2 - Set password change minimum number of days
echo -e "\n>>Password change minimum days to 7..."
sed -i 's/^PASS_MIN_DAYS.*/PASS_MIN_DAYS  7/' /etc/login.defs

##-- 7.1.3 - Set password expiring warning days
echo -e "\n>>Password expiring warning days to 7..."
sed -i 's/^PASS_WARN_AGE.*/PASS_WARN_AGE  7/' /etc/login.defs

# Also set minimum length - no CIS guidence
echo -e "\n>>Password minimum length to 6..."
sed -i 's/^PASS_MIN_LEN.*/PASS_MIN_LEN   6/' /etc/login.defs

####===============================
#### 7.2 - Disable System Accounts
####===============================
echo -e "\n---- 7.2: Disable System Accounts ----"

# Disable any system accounts that have usable shell type set
echo -e "\n>>Checking for system accounts that have a usable shell type set..."

for user in $(awk -F: '($3 < 500) {print $1 }' /etc/passwd); do
  if [ $user != "root" ]; then
    echo -e "\n->Locking system account: ${user}"
    usermod -L ${user}
    if [ $user != "sync" ] && [ $user != "shutdown" ] && [ $user != "halt" ] && [ $user != "xymon" ]; then
      echo -e "->Setting shell to /sbin/nologin for: ${user}"
      usermod -s /sbin/nologin $user
    fi
  fi
done

####==========================================
#### 7.3 - Set Default Group for root Account
####==========================================
echo -e "\n---- 7.3: Set Default Group for Root Account ----"

echo -e "\n>>Ensuring that root's GID is 0..."

roots_gid=$(grep "^root:" /etc/passwd | cut -f4 -d:)
if [ $roots_gid != "0" ]; then
  echo -e "->Setting root's GID to 0..."
  usermod -g 0 root
else
  echo -e "->Ok: root's GID is 0."
fi 

####===================================
#### 7.4 - Set Default umask for Users
####===================================
echo -e "\n---- 7.4: Set Default umask for Users ----"

echo -e "\n>>This is too restrictive and casues a lot of issues => skipping..."

# Set default UMASK for users in profile and respective shell's environment files
#for f in /etc/bashrc /etc/profile /etc/csh.cshrc /etc/zshrc /etc/csh.login /etc/kshrc; do
#  if [ -f $f ]; then
#    echo -e "\n>>Setting umask 077 in ${f}..."
#    echo "umask 077" >> $f
#  fi
#done

####===================================
#### 7.5 - Lock Inactive User Accounts
####===================================
echo -e "\n---- 7.5: Lock Inactive Local User Accounts ----"
echo -e "\n>>Setting number of days local accounts go inactive after password expire to 35 days..."
useradd -D --inactive 35

#################################################################################
#####                     Section 8 - Warning Banners                       #####
#################################################################################
echo -e "\n\n#####################################"
echo -e "#### Section 8 - Warning Banners ####"
echo -e "#####################################"

####========================================================
#### 8.1 - Set Warning Banner for Standard Login Services
#### 8.2 - Remove OS Information from Login Warning Banners
####========================================================
echo -e "\n---- 8.1/8.2: Set Warning Banner/Remove OS Info from Login Banner ----"

echo -e "\n>>Set banner contents completed in postinstall.sh => skipping..."

echo -e "\n>>Setting ownership and permisssions of /etc/issue banner..."
chown -v root:root /etc/issue
chmod -v 644 /etc/issue

echo -e "\n>>Removing other banners..."
rm -v /etc/issue.net /etc/motd

####================================
#### 8.3 - Set GNOME Warning Banner
####================================
echo -e "\n---- 8.3: Set GNOME Warning Banner ----"

echo -e "\n>>GNOME Warning Banner => skipping for now..."
# TODO - Not implemented
#----------------------------------------------------------------------
# We shouldn't be running any servers in a GUI - only workstations and test systems need apply to this?
#----------------------------------------------------------------------

# Edit /etc.d/dconf/db/gdm.d/01-banner-message and include the following:
#[org/gnome/login-screen]
#banner-message-enable=true
#banner-message-text='Type the banner message here'

# Update dconf:
#dconf update
# restart GDM to apply

#################################################################################
#####                     Section 9 - System Maintenance                    #####
#################################################################################
echo -e "\n\n########################################"
echo -e "#### Section 9 - System Maintenance ####"
echo -e "########################################"

####======================================
#### 9.1 - Verify System File Permissions
####======================================
echo -e "\n---- 9.1: Verify System File Permissions ----"

##-- 9.1.1 - Verify system file permissions
# TODO?
# Uses rpm -V to verify packages
# The recommendation here is to verify ALL packages
# This is unscored due to how long this takes
# Example: rpm -V $(rpm -qf /etc/passwd)

##-- 9.1.2 - Verify permissions on /etc/passwd
echo -e "\n>>Verifying /etc/passwd permissions..."
chmod -v 644 /etc/passwd

##-- 9.1.3 - Verify permissions on /etc/shadow
echo -e "\n>>Verifying /etc/shadow permissions..."
chmod -v 000 /etc/shadow

##-- 9.1.4 - Verify permissions on /etc/gshadow
echo -e "\n>>Verifying /etc/gshadow permissions..."
chmod -v 000 /etc/gshadow

##-- 9.1.5 - Verify permissions on /etc/group
echo -e "\n>>Verifying /etc/group permissions..."
chmod -v 644 /etc/group

##-- 9.1.6 - Verify user/group ownership on /etc/passwd
echo -e "\n>>Verifying /etc/passwd ownership..."
chown -v root:root /etc/passwd

##-- 9.1.7 - Verify user/group ownership on /etc/shadow
echo -e "\n>>Verifying /etc/shadow ownership..."
chown -v root:root /etc/shadow

##-- 9.1.8 - Verify user/group ownership on /etc/gshadow
echo -e "\n>>Verifying /etc/gshadow ownership..."
chown -v root:root /etc/gshadow

##-- 9.1.9 - Verify user/group ownership on /etc/group
echo -e "\n>>Verifying /etc/group ownership..."
chown -v root:root /etc/group

## TODO? All of these find files/directories section needs manual review after the commands run.
## Skipping for now.
##-- 9.1.10 - Find world writable files
# echo -e "Finding world writable files..."
# df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -type f -perm -0002

##-- 9.1.11 - Find un-owned files and directories
# echo -e "Finding un-owned files/directories..."
# df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -nouser -ls

##-- 9.1.12 - Find un-grouped files and directories
# echo -e "Finding un-grouped files and directories..."
# df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -nogroup -ls

##-- 9.1.13 - Find SUID system executables
# echo -e "Finding SUID system executables..."
# df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -type f -perm -4000 -print
# Ensure no rogue set-UID programs have been introduced
# verify integrity of returned files: rpm -V $(rpm -qf /usr/bin/sudo)

##-- 9.1.14 - Find SGID system executables
# echo -e "Finding SGID system executables..."
# df --local -P | awk {'if (NR!=1) print $6'} | xargs -I '{}' find '{}' -xdev -type f -perm -2000 -print
# Ensure no rogue set-GID programs have been introduced
# verify integrity of returned files: rpm -V $(rpm -qf /usr/bin/sudo)

####======================================
#### 9.2 - Review User and Group Settings
####======================================
echo -e "\n---- 9.2: Review User and Group Settings ----"

##-- 9.2.1 - Ensure password fields are not empty
# TODO
# cat /etc/shadow | /bin/awk -F: '($2 == "" ) { print $1 " does not have a password "}'
# lock accounts without empty passwords
# /usr/bin/passwd -l <username>

##-- 9.2.2 - Verify no legacy "+" entries exist in /etc/passwd
# TODO
# /bin/grep '^+:' /etc/passwd
# delete any returned entries

##-- 9.2.3 - Verify no legacy "+" entries exist in /etc/shadow
# TODO
# /bin/grep '^+:' /etc/shadow
# delete any returned entries

##-- 9.2.4 - Verify no legacy "+" entries exist in /etc/group
# TODO
# /bin/grep '^+:' /etc/group
# delete any returned entries

##-- 9.2.5 - Verify no UID 0 accounts exist other than root
# TODO
# /bin/cat /etc/passwd | /bin/awk -F: '($3 == 0) { print $1 }'
# delete any other than root

##-- 9.2.6 - Ensure root PATH integrity
echo -e "\n>>Checking root's PATH for integrity..."

if [ "$(echo $PATH | /bin/grep :: )" != "" ]; then 
  echo "-->Warning: Root PATH check => Empty Directory in root's PATH (::)"
fi 

if [ "$(echo $PATH | /bin/grep :$)" != "" ]; then 
  echo "-->Warning: Root PATH check => Trailing ':' in root's PATH" 
fi 

p=$(echo $PATH | /bin/sed -e 's/::/:/' -e 's/:$//' -e 's/:/ /g')
set -- $p 

while [ "$1" != "" ]; do 
  if [ "$1" = "." ]; then 
    echo "-->Warning: Root PATH check => root's PATH contains '.'" 
    shift 
    continue 
  fi 

  if [ -d $1 ]; then 
    dirperm=$(/bin/ls -ldH $1 | /bin/cut -f1 -d" ")
    if [ $(echo $dirperm | /bin/cut -c6 ) != "-" ]; then 
      echo "-->Warning: Root PATH check => Group Write permission set on directory $1" 
    fi
 
    if [ $(echo $dirperm | /bin/cut -c9 ) != "-" ]; then 
      echo "-->Warning: Root PATH check => Other Write permission set on directory $1" 
    fi

    dirown=$(ls -ldH $1 | awk '{print $3}')
    if [ "$dirown" != "root" ] ; then 
      echo "-->Warning: Root PATH check => $1 is not owned by root"
    fi 
  else 
    echo "-->Warning: Root PATH check => $1 is in root's path, but is not a directory. (Or doesn't exist)" 
  fi 
  shift 
done

##-- 9.2.7 - Check permissions on user home directories
echo -e "\n>>Checking permissions on user home directories..."

for dir in $(/bin/cat /etc/passwd | /bin/egrep -v '(root|halt|sync|shutdown)' |\
/bin/awk -F: '($8 == "PS" && $7 != "/sbin/nologin") { print $6 }'); do 

  dirperm=$(/bin/ls -ld $dir | /bin/cut -f1 -d" ") 
  if [ $(echo $dirperm | /bin/cut -c6 ) != "-" ]; then 
    echo "-->Warning: Home Dir Check => Group Write permission set on directory $dir" 
  fi 

  if [ $(echo $dirperm | /bin/cut -c8 ) != "-" ]; then 
    echo "-->Warning: Home Dir Check => Other Read permission set on directory $dir" 
  fi 

  if [ $(echo $dirperm | /bin/cut -c9 ) != "-" ]; then 
    echo "-->Warning: Home Dir Check => Other Write permission set on directory $dir" 
  fi 

  if [ $(echo $dirperm | /bin/cut -c10 ) != "-" ]; then 
    echo "-->Warning: Home Dir Check => Other Execute permission set on directory $dir" 
  fi 
done

##-- 9.2.8 - Check user dot file permissions
echo -e "\n>>Checking permissions on 'dot' files..."

for dir in $(/bin/cat /etc/passwd | /bin/egrep -v '(root|sync|halt|shutdown)' | /bin/awk -F: '($7 != "/sbin/nologin") { print $6 }'); do 
  for file in $dir/.[A-Za-z0-9]*; do 

    if [ ! -h "$file" -a -f "$file" ]; then 
      fileperm=$(/bin/ls -ld $file | /bin/cut -f1 -d" ")

      if [ $(echo $fileperm | /bin/cut -c6 ) != "-" ]; then 
        echo "-->Warning: Dot File Check => Group Write permission set on file $file" 
      fi
 
      if [ $(echo $fileperm | /bin/cut -c9 ) != "-" ]; then 
        echo "-->Warning: Dot File Check => Other Write permission set on file $file"
      fi
    fi 
  done 
done

##-- 9.2.9 - Check permissions on user .netrc files
# TODO - Not implemented
# .netrc files may contain unencrypted passwords
# 9.2.18 checks for existence of .netrc files

##-- 9.2.10 - Check for Presence of User .rhosts Files
echo -e "\n>>Checking for presense of .rhosts files..."

for dir in $(/bin/cat /etc/passwd | /bin/egrep -v '(root|halt|sync|shutdown)' | /bin/awk -F: '($7 != "/sbin/nologin") { print $6 }'); do 
  for file in $dir/.rhosts; do 
    if [ ! -h "$file" -a -f "$file" ]; then 
      echo "-->Warning: .rhosts file in $dir" 
    fi 
  done 
done

##-- 9.2.11 - Check Groups in /etc/passwd
echo -e "\n>>Comparing groups in /etc/passwd and /etc/group..."

for i in $(cut -s -d: -f4 /etc/passwd | sort -u ); do 
  grep -P "^.*?:x:$i:" /etc/group > /dev/null
  if [ $? -ne 0 ]; then 
    echo "-->Warning: Group $i is referenced by /etc/passwd but does not exist in /etc/group" 
  fi 
done

##-- 9.2.12 - Check That Users Are Assigned Valid Home Directories
echo -e "\n>>Checking for valid user /home/ direcotries..."

cat /etc/passwd | awk -F: '{ print $1 " " $3 " " $6 }' | while read user uid dir; do 
  if [ $uid -ge 500 -a ! -d "$dir" -a $user != "nfsnobody" ]; then 
    echo "-->Warning: The home directory ($dir) of user $user does not exist." 
  fi 
done

##-- 9.2.13 - Check User Home Directory Ownership
echo -e "\n>>Checking user home directory ownership..."

cat /etc/passwd | awk -F: '{ print $1 " " $3 " " $6 }' | while read user uid dir; do 
  if [ $uid -ge 500 -a -d "$dir" -a $user != "nfsnobody" ]; then 
    owner=$(stat -L -c "%U" "$dir") 
    if [ "$owner" != "$user" ]; then 
      echo "-->Warning: The home directory ($dir) of user $user is owned by $owner." 
    fi
  fi 
done

##-- 9.2.14 - Check for Duplicate UIDs
echo -e "\n>>Checking for duplicate User IDs..."

/bin/cat /etc/passwd | /bin/cut -f3 -d: | /bin/sort -n | /usr/bin/uniq -c | while read x ; do 
  [ -z "${x}" ] && break 
  set - $x 
  if [ $1 -gt 1 ]; then 
    users=$(/bin/gawk -F: '($3 == n) { print $1 }' n=$2 /etc/passwd | /usr/bin/xargs)
    echo "-->Warning: Duplicate UID ($2): ${users}" 
  fi 
done

##-- 9.2.15 - Check for Duplicate GIDs
echo -e "\n>>Checking for duplicate Group IDs..."

/bin/cat /etc/group | /bin/cut -f3 -d":" | /bin/sort -n | /usr/bin/uniq -c | while read x ; do 
  [ -z "${x}" ] && break 
  set - $x 
  if [ $1 -gt 1 ]; then 
    grps=$(/bin/gawk -F: '($3 == n) { print $1 }' n=$2 /etc/group | xargs) 
    echo "-->Warning: Duplicate GID ($2): ${grps}" 
  fi 
done

##-- 9.2.16 - Check that Reserved UIDs are assigned to System Accounts
echo -e "\n>>Checking reserved UIDs for user accounts..."

system_users="root bin daemon adm lp sync shutdown halt mail news uucp operator games gopher ftp nobody nscd vcsa rpc mailnull smmsp pcap ntp dbus avahi ssh rpcuser nfsnobody haldaemon avahi-autoipd distcache apache oprofile webalizer dovecot squid named xfs gdm sabayon usbmuxd rtkit abrt saslauth pulse postfix tcpdump"

/bin/cat /etc/passwd | /bin/awk -F: '($3 < 1000) { print $1" "$3 }' | while read user uid ; do
  found=0
  for tUser in ${system_users}; do
    if [ ${user} == ${tUser} ]; then
      found=1
    fi
  done

  if [ ${found} -eq 0 ]; then
    echo "-->Warning: Verify that user (${user}), which has a reserved UID (${uid}), is a system account."
  fi
done

##-- 9.2.17 - Check for Duplicate User Names
echo -e "\n>>Checking for duplicate User Names..."

cat /etc/passwd | cut -f1 -d":" | /bin/sort -n | /usr/bin/uniq -c | while read x ; do 
  [ -z "${x}" ] && break 
  set - $x 
  if [ $1 -gt 1 ]; then 
    uids=$(/bin/gawk -F: '($1 == n) { print $3 }' n=$2 /etc/passwd | xargs)
    echo "-->Warning: Duplicate User Name ($2): ${uids}" 
  fi 
done

##-- 9.2.18 - Check for Duplicate Group Names
echo -e "\n>>Checking for duplicate Group Names..."

cat /etc/group | cut -f1 -d":" | /bin/sort -n | /usr/bin/uniq -c | while read x ; do 
  [ -z "${x}" ] && break 
  set - $x 
  if [ $1 -gt 1 ]; then 
    gids=$(/bin/gawk -F: '($1 == n) { print $3 }' n=$2 /etc/group | xargs)
    echo "-->Warning: Duplicate Group Name ($2): ${gids}" 
  fi 
done

##-- 9.2.19 - Check for Presence of User .netrc Files
echo -e "\n>>Checking for presence of .netrc files..."

for dir in $(/bin/cat /etc/passwd | /bin/awk -F: '{ print $6 }'); do 
  if [ ! -h "$dir/.netrc" -a -f "$dir/.netrc" ]; then 
    echo "-->Warning: .netrc file $dir/.netrc exists"
  fi 
done

##-- 9.2.20 - Check for Presence of User .forward Files
echo -e "\n>>Checking for presence of .forward files..."

for dir in $(/bin/cat /etc/passwd | /bin/awk -F: '{ print $6 }'); do 
  if [ ! -h "$dir/.forward" -a -f "$dir/.forward" ]; then 
    echo "-->Warning: .forward file $dir/.forward exists" 
  fi 
done

#################################################################################
#######                 Section X - Additional Security                  ########
#################################################################################
echo -e "\n\n##############################################"
echo -e "###### Section X - Additional Security #######"
echo -e "##############################################"

####======================================
#### x.1 - Lock Down Permissions
####======================================

##-- x.1.1 - Set /var/lib/nfs permissions to 750  
echo -e "\n>>Setting /var/lib/nfs permissions..."
chmod -v 750 /var/lib/nfs

##-- x.1.2 - Set all /home dirs to 700 
echo -e "\n>>Setting permissions on /home..."
chmod -v 700 /home/*

##-- x.1.3 - Make sure pam_mkhomedir umask is set to 0077
echo -e "\n>>Setting pam_mkhomedir umask to 0077..."
sed -i -r -e "/pam_mkhomedir.so/s/(umask=)[0-9]+/\10077/" /etc/pam.d/system-auth-ac

##-- x.1.4 - Permissions for wtmp utmp
echo -e "\n>>Setting ownership and permissions for wtmp and utmp..."
chown -v root:root /var/log/wtmp /var/run/utmp
chmod -v 644 /var/log/wtmp /var/run/utmp

##-- x.1.5 - Set all log files in /var/log to 600
echo -e "\n>>Setting permissions for /var/log/ files..."
for FILE in $(find /var/log -type f)
do
  chmod -v 600 ${FILE}
done

# Give group read/write back for certain logs
echo -e "\n>>Additional permissions for some /var/log/ files..."
chmod -v g+rw /var/log/freshclam.log

####======================================
#### x.2 - Lock Down Ownership
####======================================

##-- x.2.1 - Change owner to root for cups log(s) 
echo -e "\n>>Looking for cups error_log to set owner..."
if [ -f /var/log/cups/error_log ]; then
  echo -e "->Setting owner for cups logs..."
  chown -v root:sys /var/log/cups/error_log*
else
  echo -e "->Nothing to do => /var/log/cups/error_log does not exist."
fi

####======================================
#### x.3 - /etc/sysctl.conf Config
####======================================
echo -e "\n>>Configuring /etc/sysctl.conf..."

cat >> /etc/sysctl.conf <<EOF
####========================================
#### sysctl.conf
#### Generated from: security-lockdown.sh
####========================================

#### Kernel Tuning ####

# CIS 1.6.2 - sysctl kernel.randomize_va_space
kernel.randomize_va_space = 2

# Disable Magic sysrq keys
kernel.sysrq = 0

#### File System ####

# CIS 1.6.1 - sysctl fs.suid_dumpable
fs.suid_dumpable = 0

#### Network Tuning ####

## Network Global Settings ##

# IPv4 Default
# CIS 4.2.2 - net.ipv4.conf.default.accept_redirects
net.ipv4.conf.default.accept_redirects = 0
# CIS 4.2.1 - net.ipv4.conf.default.accept_source_route
net.ipv4.conf.default.accept_source_route = 0
# CIS 4.2.4 - net.ipv4.conf.default.log_martians
net.ipv4.conf.default.log_martians = 1
# CIS 4.2.7 - net.ipv4.conf.default.rp_filter
net.ipv4.conf.default.rp_filter = 1
# CIS 4.2.3 - net.ipv4.conf.default.secure_redirects
net.ipv4.conf.default.secure_redirects = 0
# CIS 4.1.2 - net.ipv4.conf.default.send_redirects
net.ipv4.conf.default.send_redirects = 0

# IPv4 All Interfaces
# CIS 4.2.2 - net.ipv4.conf.all.accept_redirects
net.ipv4.conf.all.accept_redirects = 0
# CIS 4.2.1 - net.ipv4.conf.all.accept_source_route
net.ipv4.conf.all.accept_source_route = 0
# CIS 4.2.4 - net.ipv4.conf.all.log_martians
net.ipv4.conf.all.log_martians = 1
# CIS 4.2.7 - net.ipv4.conf.all.rp_filter
net.ipv4.conf.all.rp_filter = 1
# CIS 4.2.3 - net.ipv4.conf.all.secure_redirects
net.ipv4.conf.all.secure_redirects = 0
# CIS 4.1.2 - net.ipv4.conf.all.send_redirects
net.ipv4.conf.all.send_redirects = 0

# IPv4 ICMP
# CIS 4.2.5 - net.ipv4.icmp_echo_ignore_broadcasts
net.ipv4.icmp_echo_ignore_broadcasts = 1
# CIS 4.2.6 - net.ipv4.icmp_ignore_bogus_error_responses
net.ipv4.icmp_ignore_bogus_error_responses = 1

# IPv4 IP
# CIS 4.1.1 - Disable IP Forwarding
net.ipv4.ip_forward = 0

# IPv4 TCP
net.ipv4.tcp_max_syn_backlog=4096
net.ipv4.tcp_sack = 1
# CIS 4.2.8 - net.ipv4.tcp_syncookies
net.ipv4.tcp_syncookies = 1
net.ipv4.tcp_timestamps = 1
net.ipv4.tcp_window_scaling = 1
# Increase Linux autotuning TCP buffer limits
# min, default, and max number of bytes to use
net.ipv4.tcp_rmem= 10240 87380 12582912
net.ipv4.tcp_wmem= 10240 87380 12582912

# Network Core/IPv4 Gigabit tweaks
net.core.rmem_max=12582912
net.core.wmem_max=12582912
EOF

echo -e "\n>>Ensuring proper ownership and permissions of /etc/sysctl.conf..."
chown -v root:root /etc/sysctl.conf
chmod -v 600 /etc/sysctl.conf

echo -e "\n>>Loading /etc/sysctl.conf changes..."
sysctl -p /etc/sysctl.conf

####========================================
#### x.4 - /etc/security/limits.conf 
####========================================
echo -e "\n>>Configuring /etc/security/limits.conf..."

cat > /etc/security/limits.conf <<EOF
####========================================
#### limits.conf
#### Generated from: security-lockdown.sh
####========================================
#<domain>        <type>  <item>  <value>
#
#Where:
#<domain> can be:
#        - an user name
#        - a group name, with @group syntax
#        - the wildcard *, for default entry
#        - the wildcard %, can be also used with %group syntax,
#                 for maxlogin limit
#
#<type> can have the two values:
#        - "soft" for enforcing the soft limits
#        - "hard" for enforcing hard limits
#
#<item> can be one of the following:
#        - core - limits the core file size (KB)
#        - data - max data size (KB)
#        - fsize - maximum filesize (KB)
#        - memlock - max locked-in-memory address space (KB)
#        - nofile - max number of open files
#        - rss - max resident set size (KB)
#        - stack - max stack size (KB)
#        - cpu - max CPU time (MIN)
#        - nproc - max number of processes
#        - as - address space limit (KB)
#        - maxlogins - max number of logins for this user
#        - maxsyslogins - max number of logins on the system
#        - priority - the priority to run user process with
#        - locks - max number of file locks the user can hold
#        - sigpending - max number of pending signals
#        - msgqueue - max memory used by POSIX message queues (bytes)
#        - nice - max nice priority allowed to raise to values: [-20, 19]
#        - rtprio - max realtime priority
#
# CIS 1.6.1 (restrict core dumps) - hard core 0
*               soft    core            0
*               hard    core            0
*               soft    nproc          1024
*               hard    nproc          2048
*               soft    nofile         1024
*               hard    nofile         2048
EOF

#====================================================================
# Security Lockdown Script Completed
#====================================================================
echo -e "\n\n#=================================================================="
echo "# Security Lockdown Completed. - A reboot is recommended."
echo "#=================================================================="
exit 0
